/****************************** Start of GPL Block ****************************
 *   ImageHost Grabber - Imagegrabber is a firefox extension designed to 
 *   download pictures from image hosts such as imagevenue, imagebeaver, and 
 *   others (see help file for a full list of supported hosts).
 *
 *   Copyright (C) 2007   Matthew McMullen.
 * 
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ***************************  End of GPL Block *******************************/


/* ProgressListener class:  This is the stream listener class for the channels used to download
 * our files.  It is primarily responsible for sending the downloaded chunks to the corresponding
 * files as the data becomes availble.  Also, It is primarily responsible for monitoring the status 
 * and progress of our downloads.
 * 
 * aBuffer is a buffered output stream to the corresponding file.  It should be an implementation
 * nsIBufferedOutputStream class.
 */

// For an example of how a stream/progress listener should be set up for typical
// http connections, check out: 
// https://developer.mozilla.org/En/Creating_Sandboxed_HTTP_Connections

function ProgressListener(aBuffer) {
	this.buffer = aBuffer;

	this.refURL = "";
	this.reqObj = new Object();
	this.aFile = new Object();

	this.fileGood = "";

	this.fileContents = "";

	this.SIS = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);

	this.resumable = true;

	this.request = new Object();
	}

ProgressListener.prototype = {
	checkFile : function() {
		var fileURI = ioService.newFileURI(this.aFile);
		var channel = ioService.newChannelFromURI(fileURI);

		var stream = channel.open();

		try {
			var count = stream.available();
			this.SIS.init(stream);
			var shitty = this.SIS.read(count);			

			this.fileContents = shitty;

			// For jpegs, the first two bytes are the start of image (SOI) marker.
			// the SOI marker is FFD8
			if( shitty.match(/\xFF\xD8/) ) this.fileGood = "yes";
			// For pngs, the first four bytes are 8950 4E47
			else if( shitty.match(/\x89\x50\x4E\x47/) ) this.fileGood = "yes";
			// For bmps, the first two bytes are 424D
			else if ( shitty.match(/^\x42\x4D/) ) this.fileGood = "yes";
			// For gifs, the first six bytes are 4749 4638 3961
			// Who in their right mind would be downloading gifs?
			else if ( shitty.match(/^GIF89a/) ) this.fileGood = "yes";
			// For flvs, the first three bytes are 464C56 (FLV)
			else if ( shitty.match(/^\x46\x4C\x56/) ) this.fileGood = "yes";

			else this.fileGood = "no";
			}
		catch(e) { this.fileGood = "yes"; }
		},

	// nsIInterfaceRequestor
	getInterface: function (aIID) {
		try {
			return this.QueryInterface(aIID);
			}
		catch (e) {
			throw Components.results.NS_NOINTERFACE;
			}
		},

	retry : function() {
		var toDieOrNot = prefManager.getBoolPref("extensions.imagegrabber.killmenow");
		if (toDieOrNot && !this.override_stop) {
			LOG("In function requestObj.retry, received the call to die!\n");
			return; }

		this.request.cancel(Components.results.NS_OK);
	},

	onStartRequest : function(request, context){
		var toDieOrNot = prefManager.getBoolPref("extensions.imagegrabber.killmenow");
		if (toDieOrNot && !this.reqObj.override_stop) return;
		if (this.reqObj.aborted) return;

		if (request.status == Components.results.NS_ERROR_NOT_RESUMABLE) this.resumable = false;

		updateDownloadProgress(null, this.reqObj.uniqID, this.aFile.leafName, null, "Starting Download.");
		this.reqObj.fileName = this.aFile.leafName;

		this.request = request;

		var req_timeout = reqTimeout;  // timeout is in milliseconds
		this.callwrapper = new CCallWrapper(this, req_timeout, 'retry', "download of " + this.reqObj.uniqID);
		CCallWrapper.asyncExecute(this.callwrapper);
		},

	onStopRequest : function(request, context){
		this.buffer.close();

		this.callwrapper.cancel();

		var toDieOrNot = prefManager.getBoolPref("extensions.imagegrabber.killmenow");
		if (toDieOrNot && !this.reqObj.override_stop) return;
		if (this.reqObj.aborted) return;

		aStatus = request.status;

		if(aStatus == 0) 	{
			if (this.reqObj.curProgress < this.reqObj.maxProgress) {
					var retry_dick = "(" + String(maxRetries - this.reqObj.retryNum + 1) + " of " + String(maxRetries) + ")";
					updateDownloadProgress(null, this.reqObj.uniqID, null, null, "Download did not complete!  Retrying " + retry_dick)
					LOG("In onStopRequest, file did not download correctly, " + this.refURL + ", " + this.reqObj.uniqID + ", Retrying!\n");
					this.reqObj.overwrite = true;
					this.reqObj.retry();
					return;
					}

			this.checkFile();
			if(this.fileGood == "no") {
				if(this.fileContents.match(/Error: Unavailable/)) {
					updateDownloadProgress(null, this.reqObj.uniqID, null, 100, "Error from server.  Not available!");
					}
				else {
					var retry_dick = "(" + String(maxRetries - this.reqObj.retryNum + 1) + " of " + String(maxRetries) + ")";
					updateDownloadProgress(null, this.reqObj.uniqID, null, null, "File Not Good!  Retrying " + retry_dick)
					LOG("File did not download correctly, " + this.refURL + ", " + this.reqObj.uniqID + ", Retrying!");
					this.reqObj.overwrite = true;
					this.reqObj.curProgress = 0;
					this.reqObj.maxProgress = 0;
					this.reqObj.retry();
					return;
					}
				}
			else updateDownloadProgress(null, this.reqObj.uniqID, null, null, "Finished!");

			clearFromWin(this.reqObj.uniqID);

			this.reqObj.finished = true;
			this.reqObj.inprogress = false;
			this.reqObj.queueHandler();
			}

		if(aStatus != 0) {
			LOG("onStateChange has resulted in a non-successful status code inside of the persist progress listener.\n");
			LOG("In onStateChange, this.refURL is equal to" + this.refURL + "\n");
			LOG("In onStateChange, request.name is equal to " + request.name + "\n");
			LOG("In onStateChange, aStatus: " + aStatus + "\n");

			var retry_dick = "(" + String(maxRetries - this.reqObj.retryNum + 1) + " of " + String(maxRetries) + ")";

			updateDownloadProgress(null, this.reqObj.uniqID, null, null, "File Not Good!  Retrying " + retry_dick)
			if (aStatus == Components.results.NS_ERROR_NOT_RESUMABLE) this.reqObj.notResumable = true;
			this.reqObj.overwrite = true;
			this.reqObj.curProgress = 0;
			this.reqObj.maxProgress = 0;
			this.reqObj.retry();
			}
		},

	// nsIChannelEventSink
	onChannelRedirect: function (aOldChannel, aNewChannel, aFlags) {
		LOG("Entered onChannelRedirect...\n");
		},

	onDataAvailable : function(request, context, stream, offset, count) { 
		this.reqObj.curProgress += count;

		this.callwrapper.cancel();

		while(count > 0) count -= this.buffer.writeFrom(stream, count);

		var curProgress = this.reqObj.curProgress;
		var maxProgress = this.reqObj.maxProgress;

		var toDieOrNot = prefManager.getBoolPref("extensions.imagegrabber.killmenow");
		if ((toDieOrNot && !this.reqObj.override_stop) || this.reqObj.aborted) {
			request.cancel(Components.results.NS_OK);
			try { this.buffer.close(); }
			catch(e) {}
			return;
			}

		updateDownloadProgress(null, this.reqObj.uniqID, null, (curProgress/maxProgress) * 100, "Downloading " + curProgress + " of " + maxProgress + "...");

		var req_timeout = reqTimeout;  // timeout is in milliseconds
		this.callwrapper = new CCallWrapper(this, req_timeout, 'retry', "download of " + this.reqObj.uniqID);
		CCallWrapper.asyncExecute(this.callwrapper);
		},

	// nsIProgressEventSink (not implementing will cause annoying exceptions)
	onProgress : function (aRequest, aContext, aProgress, aProgressMax) { 
		if (this.reqObj.maxProgress == 0) this.reqObj.maxProgress = aProgressMax;
		if (this.reqObj.maxProgress == -1) this.request.cancel(Components.results.NS_ERROR_ABORT);
		},

	onStatus : function (aRequest, aContext, aStatus, aStatusArg) { },

	// nsIHttpEventSink (not implementing will cause annoying exceptions)
	onRedirect : function (aOldChannel, aNewChannel) { },

	// we are faking an XPCOM interface, so we need to implement QI
	QueryInterface : function(aIID) {
		if (aIID.equals(Components.interfaces.nsISupports) ||
			aIID.equals(Components.interfaces.nsIInterfaceRequestor) ||
			aIID.equals(Components.interfaces.nsIChannelEventSink) || 
			aIID.equals(Components.interfaces.nsIProgressEventSink) ||
			aIID.equals(Components.interfaces.nsIHttpEventSink) ||
			aIID.equals(Components.interfaces.nsIStreamListener)) return this;

			throw Components.results.NS_NOINTERFACE;
  		}
	}