/****************************** Start of GPL Block ****************************
 *   ImageHost Grabber - Imagegrabber is a firefox extension designed to 
 *   download pictures from image hosts such as imagevenue, imagebeaver, and 
 *   others (see help file for a full list of supported hosts).
 *
 *   Copyright (C) 2007   Matthew McMullen.
 * 
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ***************************  End of GPL Block *******************************/


/* requestObj class:  It's necessary to  create a class that encompasses the XMLHttpRequest
 * class.  Each new instance of requestObj will create a new instance of XMLHttpRequest.
 * What this does is it allows the XMLHttpRequest object to access data that is not part
 * of the XMLHttpRequest class definition.  The difference between this and adding a property
 * or member on the fly (i.e. object.whatever = somestuff), is that the data is guaranteed to 
 * be accessible using this technique.  Whereas, when adding members on the fly, XMLHttpRequest
 * had a bad tendency of losing the data that was assigned to it on the fly.
 */
function requestObj() {
	// These are the properties that need to be set (by someone) for each instance of the class
	this.uniqID = "";
	this.reqURL = "";
	this.hostFunc = new Function();
	this.regexp = null;
	this.retryNum = 0;
	this.dirSave = "";
	this.fileName = "";
	this.baseFileName = "";
	this.pageNum = 0;
	this.totLinkNum = 0;
	this.curLinkNum = 0;
	this.uniqFN_prefix = "";
	this.status = "";
	this.referer = "";

	// everything else below is blah blah blah
	this.callwrapper = new Object();

	this.cp = this.constructor.prototype; // make a convenient shortcut to the prototype

	this.finished = false;
	this.inprogress = false;
	this.curProgress = 0;
	this.maxProgress = 0;
	this.aborted = false;
	this.overwrite = false;
	this.override_stop = false;
	this.stopped = false;
	this.retried = false;
	this.notResumable = false;

	this.nextRequest = new Object();
	this.previousRequest = new Object();
	this.firstRequest = new Object();
	this.lastRequest = new Object();

	this.progListener = new Object();

	this.xmlhttp = new XMLHttpRequest();
	this.xmlhttp.parent = this;

	}


requestObj.prototype = {
	curThread : 0,

	constructor : requestObj,

	debugLog : function req_debugLog () {
		if (!debugOut) return;

		function var_out(a,b,c) { 
			LOG("In requestObj with uniqID of " + this.uniqID +
			    ", Property: " + a + ", Old Val: " + b + ", New Val: " + c + "\n");
			return c;
			}
		this.watch('reqURL', var_out);
		this.watch('retryNum', var_out);
		this.watch('curLinkNum', var_out);
		this.watch('finished', var_out);
		this.watch('inprogress', var_out);
		},

	abort : function req_abort(additional_message) {
		var retryURL = this.reqURL;

		if(this.finished == true) return;

		updateDownloadProgress(null, this.uniqID, null, null, "Request Aborted!  " + (additional_message?additional_message:""));
		LOG(retryURL + " has been aborted.\n");

		this.inprogress = false;
		this.finished = true;
		this.aborted = true;
		this.callwrapper.cancel();
		this.xmlhttp.abort();


		var toDieOrNot = prefManager.getBoolPref("extensions.imagegrabber.killmenow");
		if (toDieOrNot) {
			LOG("In function requestObj.retry, received the call to die!\n");
			return; }

		this.queueHandler();
		},


	retry : function req_retry() {
		LOG("Entering function requestObj.retry\n");

		var toDieOrNot = prefManager.getBoolPref("extensions.imagegrabber.killmenow");
		if (toDieOrNot && !this.override_stop) {
			LOG("In function requestObj.retry, received the call to die!\n");
			return; }

		this.inprogress = false;
		var retryURL = this.reqURL;
		LOG("In function requestObj.retry, this.retryNum is equal to: " + this.retryNum + "\n");

		if(this.retryNum < 1) {
			updateDownloadProgress(null, this.uniqID, null, null, "Request Failed!  Too many retries.");
			LOG(this.uniqID + " has been retried the maximum number of times... we're gonna give up.\n");

			this.finished = true;
			this.aborted = true;

			this.queueHandler();
			return;
			}

		this.finished = false;
		this.aborted = false;

		LOG("In function req_retry, fixing to execute xmlhttp.abort\n");		
		this.xmlhttp.abort();

		this.retryNum--;

		var retry_dick = "(" + String(maxRetries - this.retryNum + 1) + " of " + String(maxRetries) + ")";
		updateDownloadProgress(null, this.uniqID, this.reqURL, null, "Re-Starting HTTP Request..." + retry_dick);

		this.retried = true;
		this.init();

		LOG("Exiting function requestObj.retry\n");
		},

	errHandler : function req_errHandler(event) {
		LOG("Entering function requestObj.errHandler\n");

		req = this.parent;
		var retryURL = req.reqURL;

		req.inprogress = false;

		var toDieOrNot = prefManager.getBoolPref("extensions.imagegrabber.killmenow");
		if (toDieOrNot) {
			LOG("In function requestObj.retry, received the call to die!\n");
			return; }

		req.callwrapper.cancel();

		if(req.retryNum < 1) {
			updateDownloadProgress(null, req.uniqID, null, null, "Request Failed!  Too many retries.");
			LOG(retryURL + " has been retried the maximum number of times... we're gonna give up.\n");

			req.finished = true;
			req.aborted = true;

			req.queueHandler();
			return;
			}

		this.finished = false;
		this.aborted = false;

		LOG(retryURL + " has encountered an error.  Retrying.\n");
		var retry_dick = "(" + String(maxRetries - req.retryNum + 1) + " of " + String(maxRetries) + ")";
		updateDownloadProgress(null, req.uniqID, null, null, "Error In Request!  Retrying. " + retry_dick);

		req.retryNum--;

		req.init()

		LOG("Exiting function requestObj.errHandler\n");
		},


	getNextAvailable : function req_getNextAvailable(req_obj) {
		var next_obj = req_obj.firstRequest;
		var not_found = true;
		while(not_found) {
			if(next_obj == null) not_found = false;
			else if(next_obj.inprogress == false && next_obj.finished == false) not_found = false;
			else next_obj = next_obj.nextRequest;
			}
		return next_obj;
		},

	queueHandler : function req_queueHandler() {
		var toDieOrNot = prefManager.getBoolPref("extensions.imagegrabber.killmenow");
		if (toDieOrNot) return;

		this.cp.curThread = this.countThreads();

		var next_req = this.getNextAvailable(this.firstRequest);

		if (next_req) {
			next_req.init();
			}
		else {
			if(this.curThread == 0){
				updateDownloadStatus("all done....");
				}
			}
		},

	countThreads : function req_countThreads() {
		var numThreads = 0;
		var firstReq = this.firstRequest;
		var nextReq = firstReq.nextRequest;

		if (firstReq.inprogress) numThreads++;
		while (nextReq) {
			if (nextReq.inprogress) numThreads++;
			nextReq = nextReq.nextRequest;
			}

		return numThreads;
		},

	init : function req_init() {
		LOG("Entering function requestObj.init with uniqID of: " + this.uniqID + "\n");

		this.cp.curThread = this.countThreads();
		if(this.cp.curThread >= maxThreads)	{
			LOG("In function requestObj.init with uniqID of: " + this.uniqID + ", curThread >= maxThreads\n");
			return;
			}	
		if(this.finished || this.inprogress) {
			LOG("In function requestObj.init with uniqID of: " + this.uniqID + 
			    "\nfinished: " + this.finished + ", inprogress: " + this.inprogress + "\n");
			return;
			}

		this.inprogress = true;

		if (!this.retried) updateDownloadProgress(null, this.uniqID, this.reqURL, null, "Starting HTTP Request...");

		var req_timeout = reqTimeout;  // timeout is in milliseconds

		this.callwrapper = new CCallWrapper(this, req_timeout, 'retry', this.uniqID);
		CCallWrapper.asyncExecute(this.callwrapper);

		this.xmlhttp.open("GET", this.reqURL, true);
		if (this.referer) this.xmlhttp.setRequestHeader("Referer", this.referer);
		this.xmlhttp.onreadystatechange = this.init2;
		this.xmlhttp.onload = this.hostFunc;
		this.xmlhttp.onerror = this.errHandler;
		this.xmlhttp.send(null);

		LOG("Exiting function requestObj.init with uniqID of:" + this.uniqID + "\n");
		},

	init2 : function req_init2() {
		var myself = String(arguments.callee).match(/(function.*)\(.*\)[\n\s]*{/m)[1];
		LOG("Entering " + myself + "\n");

		var req = this.parent;

		LOG("In " + myself + ", this.readyState: " + this.readyState + "\n");

		if (this.readyState == 2) {
			var contType = this.getResponseHeader("Content-type");
			if (!contType) return;
			if (contType.match(/image\/.+/)) {
				this.onload = null;
				req.callwrapper.cancel();
				this.abort();
				doStartDownload(req, req.reqURL);
				updateDownloadProgress(null, req.uniqID, req.reqURL, null, "Starting Download...");
				return;
				}
			}
		}

	} // end of class prototype